;
; RAW DISK DUMP UTILITY FOR C3-OEM
;
; DUMPS THE CONTENTS OF A DISK AND SENDS OUTPUT
; TO THE SERIAL PORT AS ASCII HEX. IT WILL
; CAPTURE ALL DATA ON THE DISK INCLUDING THE
; TRACK/SECTOR FORMATTING INFORMATION.
;
; DUMPS EVERY BYTE ENCOUNTERED ON A TRACK
; STARTING AT THE INDEX PULSE.
;
; NOTE: DOS FORMATTED TRACKS WILL CONTAIN
; SPURIOUS CHARACTERS BEFORE AND AFTER SECTORS
; AND CAN BE IGNORED.
;
; NOTE 65U DISKS SWITCH ENCODING FROM 8E1 TO 8N1
; ON TRACKS 2-77 AFTER TRACK IDENTIFIER

;BASED ON ED'S OSIDUMP UTILITY
;
;

ORG	=	$0200

MAXTRK8 =   77	; # Tracks to read 8 disk
MAXPAG8 =   15	; # Pages to read 8 disk
MAXPAGH =   16  ; # Pages HD
FILLCH	=	$00	; FILL CHARACTER

;BUFFER	=	$1000	; TRACK BUFFER (to $1F00 8") (to 1FFF HD)
DDPIA	=	$C000	; DISK CONTROLLER PIA
DDACIA  =   $C010   ; DISK CONTROLLER ACIA

;Control Character Constants XMODEM etc.
CR		=	$0D
LF		=	$0A
SOH		=	$01		; start block marker
STOX    =   $02     ; start of 1K block marker (STX is reserved)
EOT		=	$04		; end of text marker
ACK		=	$06		; good block acknowledged
NAK		=	$15		; bad block acknowledged
CAN		=	$18		; cancel (not standard, not supported)
ESC		=	$1B		; ESC to exit




;
; XMODEM storage
;
;
xmdata      =   $10
lastblk		=	xmdata			; flag for last block
blkno		=	xmdata+1		; block number 
Xerrcnt		=	xmdata+2		; error counter 10 is the limit
crc			=	xmdata+3		; CRC lo byte  (two byte variable)
crch		=	xmdata+4		; CRC hi byte  
ptr			=	xmdata+5		; data pointer (two byte variable)
ptrh		=	xmdata+6		;   "    "
eofp		=	xmdata+7		; end of file address pointer (2 bytes)
eofph		=	xmdata+8		;  "	"	"	"
Xretry		=	xmdata+9		; retry counter 
Xretry2		=	xmdata+10		; 2nd counter
lastdata    =   xmdata+11       ;flag buffer of file
Xsavem      =   xmdata+12       ;saved index to resume filling send buffer

;storage space
STORE   =   $20   ;storage
MAXTRK  =   STORE+01 ;40 for 5.25, 77 for 8, 255 for HD
MAXTRH  =   STORE+02 ;high byte max track for HD, 0 for floppy
MAXPAG  =   STORE+03 ;9 for 5.25, 15 for 8, 16 for HD
ACIACTL =   STORE+04 ;controls cassette/rs-232 ACIA divisor, format
DRVNUM  =   STORE+05 ;drive# 0-3
TRK     =   STORE+06 ;current track #
TRKHI   =   STORE+07 ;high byte current track # (block #)

;CNT	    =   STORE+09 ;
TMP     =   STORE+10

PDEC8	=   STORE+11 ;word - hex to decimal source
PDEC9	=   STORE+13 ;flag byte
PDEC10	=   STORE+14 ;3 byte result hex to decimal
USEXMOD =   STORE+17 ;
HDTRK   =   STORE+18 ;
TMODE   =   STORE+19
CURSECT  =  STORE+20
CURHEAD =   STORE+21
CURCYLN =   STORE+22
CURCYLH =   STORE+23

Rbuff =	$40 ; to $C5 temp 133 byte xmodem receive buffer

;zero page storage
ACIAVAL =   $E0	; 1 byte storage for current serial word format for ACIA 8E1/8N1
TRKCNT  =   $E1 ; 1 bytes
PTRSTOR =   $E2 ; 2 bytes
HCNT    =   $E4 ; 1 bytes
HPAGE   =   $E5 ; 1 byte
HDATA   =   $E6 ; 2 bytes
TDATA   =   $E8 ; 2 bytes

;xmodem status return values
WRONGBLK = $FD		;xmode recv wrong blk #
MISMATCH = $FC
ABORTRECV = $FE	    ;xmode recv abort recv

; defined at end Rbuff		=	$0F00      	; temp 133 byte receive buffer 
					;(place anywhere, page aligned)


;   *=$0129	; entry point for OS65A ROM MONITOR 'GO' command
;	.byte $00,$00,$00,$00,$FD,>ORG,<ORG
; (Preceeding replaced with .EXE directive)
	
	*=	ORG
	.EXE *	;A65 emit OSI .lod start address operation
	SEI
	LDA #$15      ;B1 = 8N2 /16 RTStxIRQ  rxIRQ ;$15 = 8N1 /16 RTSNOtxIRQ NOrxIRQ; $B5 = irqs on 8N1
	STA ACIACTL   
	JSR INZACIA
	LDA #$00
	STA DRVNUM
	STA USEXMOD
	STA MAXTRH

	LDA #MAXTRK8
	STA	MAXTRK
	LDA #MAXPAG8
	STA MAXPAG


TOP
	JSR DRWMENU
MENUL0
	JSR INKEY
	CMP #$20
	BCC MENUL0
	CMP #$80
	BCS MENUL0
	JSR OUTPUT
	CMP #$31
	BNE *+5
	JMP RAWREAD
	CMP #$32
	BNE *+5
	JMP HDUMP  
	CMP #$33
	BNE *+5
	JMP	SETDRV
    CMP #$34
	BNE *+5
	JMP TDUMP
	CMP #$35
	BNE *+5
	JMP ACIADIV
	CMP #$36
	BNE *+5
	JMP SETXFER
	CMP #$37
	BNE *+5
	JMP	($FFFC)	;Exit
	LDA #'?
	JSR OUTPUT
	JMP TOP

ACIADIV
	JSR PRINT
	.BYTE $D,$A,'Set (N)ormal or 16x (F)aster? >',0
	JSR INKEY
	JSR OUTPUT
	ORA #$20
	CMP #'f
	BEQ DIV1
	CMP #'n
	BEQ DIV16
	JMP INPERR

DIV1
	LDA #$14  ;8N1 div 1

DIV16=*+1
	BIT $15A9  ;$15A9	 ;15=8N1 /16 -RTS lo no irq
	STA ACIACTL
	JSR INZACIA
	JMP TOP

SETDRV
	JSR PRINT
	.BYTE $D,$A,'Select drive (A), (B), (C), (D) ? >',0
	JSR INKEY
	JSR OUTPUT
	AND #$DF
	CMP #'A
	BCC SETERR
	CMP #'E
	BCS SETERR
	SBC #'@		;carry is clear so val -1
	STA DRVNUM
	JMP TOP
SETERR
	JMP INPERR

SETXFER
	JSR PRINT
	.BYTE $D,$A,'Use (X)modem, Xmodem(C)RC or(H)ex transfer? ? >',0
	JSR INKEY
	JSR OUTPUT
	AND #$DF
	CMP #'X
	BNE SETXF1
	LDA #$C0	;bit 7,6 set = Xmodem(Checksum)
	BNE SETXM
SETXF1
	CMP #'C
	BNE SETXF2
	LDA #$80
	BNE SETXM	;bit 7 set = Xmodem/CRC
SETXF2
	CMP #'H
	BNE SETERR ;hex
	LDA #$00
SETXM
	STA USEXMOD
	JMP TOP
INPERR
	LDA #'?
	JSR OUTPUT
	JMP TOP


;-----------------------------------------------------------
RAWREAD

	JSR	PRINT
	.BYTE	CR,LF
	.BYTE	'WRITE PROTECT ALL DISKS!',CR,LF
	.BYTE	CR,LF
	.BYTE	'INSERT DISK TO BE DUMPED',CR,LF
	.BYTE   'ESC KEY OR ^X ABORTS...',CR,LF,0

	JSR	ANYKEY

	JSR	INITPIA
	JSR SELDRV

	LDA	#0
	STA MAXTRH
	STA	TRK
	STA TRKHI
	STA lastdata
	STA TMODE

	LDA #MAXTRK8
	STA	MAXTRK
	LDA #MAXPAG8
	STA MAXPAG
	
	JSR	TZERO	; STEP TO TRACK 0
	BCS	NOTRK
	LDA #$58    ;0 10 110 00 ;$58-  recv irq, RTS HIGH no xmit irq,  8E1,  DIVIDE BY 1,
	STA ACIAVAL	

LOOP	
	LDA	TRK
	BNE LOOP3
	JSR RDTZERO
	BCS NODRIVE
	BCC LOOP5	

LOOP3
	LDA	TRK	
	CMP	MAXTRK
	BCS	LOOPBCS
	JSR	RDTRKU
	BCS NODRIVE
LOOP5
	JSR	SAVMSG
	LDA	MAXPAG   ;how many pages to read?  15/16
	JSR	SAVBUF
LOOPBCS
	BCS QUIT
	JSR FRACIANW
	BCC LOOP4
	CMP #$1B   ;ESC key?
	BEQ QUIT
	CMP #$18   ;^X
	BEQ QUIT
LOOP4
	LDA	TRK
	CMP	MAXTRK
	BEQ QUIT
	JSR	TNEXT	; STEP TO NEXT TRACK
LOOP2
	JMP	LOOP

NODRIVE
	JSR	TZERO
	JSR INITPIA
	JSR PRINT
	.BYTE CR,LF, 'ERROR Waiting for disk index!',0
	JSR ANYKEY
	JMP TOP

NOTRK
	JSR INITPIA
	JSR PRINT
	.BYTE CR,LF,CR,LF, 'ERROR Seeking Track 0',0
	JSR ANYKEY
	JMP TOP	

QUIT
	JSR	TZERO
	JSR INITPIA
	JSR	PRINT
	.BYTE	CR,LF,CR,LF
	.BYTE	'DONE ... REMOVE DISK',CR,LF,0
	JSR	INKEY
	JMP TOP

; SAVE BUFFER CONTENTS (A) AS HEX
SAVNOBUF
	JSR FRACIANW
	BCC SAVNO1
	CMP #$1B   ;ESC key?
	BEQ SAVABORT
	CMP #$18   ;^X
	BEQ SAVABORT
SAVNO1	
	CLC
	RTS

SAVBUF	
	BIT TMODE
	BMI SAVNOBUF
	PHA
	JSR	RESPTR
	PLA
	STA	TRKCNT
	LDA	#0
	;STA	CNT
	;STA	CNT+1
	TAY
	TAX
	LDA USEXMOD
	BNE SAVXMOD1
	BEQ	SAVBUF5	 ;always

SAVBUF1	
	CPX	#0
	BNE	SAVBUF2
	JSR	CRLF
SAVBUF7
	JSR FRACIANW
	BCC SAVBUFX
	CMP #$1B   ;ESC key?
	BEQ SAVABORT
	CMP #$18   ;^X
	BNE SAVBUFX

SAVABORT	
	JSR PRINT
	.BYTE $A,$D,'Abort!',0
	SEC
	RTS


SAVBUFX
	LDX	#16
	BNE	SAVBUF3

SAVBUF2	
	LDA	#' '
	JSR	OUTPUT
SAVBUF3	
	LDA	(TDATA),Y
	JSR	PHEXA
	;INC	CNT
	;BNE	SAVBUF4
	;INC	CNT+1
SAVBUF4	
	DEX
	INY
	BNE	SAVBUF1
	INC	TDATA+1
	DEC	TRKCNT
SAVBUF5	
	LDA	TRKCNT
	BNE	SAVBUF1
	JSR	CRLF
	CLC
	RTS


; SAVE BUFFER CONTENTS (A) pages USING XMODEM

SAVXMOD
	PHA	
	JSR	RESPTR
	PLA
	STA TRKCNT

SAVXMOD1
	LDA TDATA
	STA ptr
	LDA TDATA+1
	STA ptrh
	LDA MAXPAG
	CLC
	ADC TDATA+1
	STA eofph
	LDA TDATA
	STA eofp

	LDA	TRK
	CLC
	ADC #$01
	CMP	MAXTRK
	BNE SAVXMOD2
	LDA MAXTRH   ;when saving HD
	CMP TRKHI
	BNE SAVXMOD2
	LDA #$C0	 ;last block flag
	STA lastdata

SAVXMOD2
	JSR XResume

	BCC	XMODBUF5
	JSR PRINT
	.BYTE $A,$D,'Error Xmodem send!',0
	SEC
	RTS

XMODBUF5	
	RTS



; INIT POINTERS, CLEAR BUFFER
INIT	
	JSR	RESPTR	; SET POINTERS
	LDY	#0
	LDA	#FILLCH
INIT1	
	STA	(TDATA),Y
	INY
	BNE	INIT1
	INC	TDATA+1
	DEC	TRKCNT
	BNE	INIT1
	;fall through

; RESET BUFFER POINTERS
RESPTR	
	LDA	MAXPAG
	STA	TRKCNT
	LDA	#<BUFFER
	STA	TDATA
	LDA	#>BUFFER
	STA	TDATA+1
	RTS

; INIT DISK CONTROLLER PIA
INITPIA	
	LDY	#0
	LDA #$40
	STY	DDPIA+1	; SELECT DDRA
	STA	DDPIA	; SET PORTA TO ALL INPUTS except PA6
	LDX	#4		; 0000 0100
	STX	DDPIA+1	; SELECT PORTA
	STA DDPIA    ; SET PB6 HIGH
	STY	DDPIA+3	; SELECT DDRB
	DEY
	STY	DDPIA+2	; SET PORTB TO ALL OUTPUTS
	STX	DDPIA+3	; SELECT PORTB
	STY	DDPIA+2	; SET PORTB OUTPUTS HIGH
	RTS


;SELECT DRIVE PB5 PA6  DRIVE (1-4)
;              0   0	#4
;              0   1	#3
;              1   0	#2
;              1   1	#1

SELDRV
	LDA DRVNUM  ;0-3
	LSR A
	TAY
	BCC *+5
	LDA #$00
	BIT $40A9
	STA DDPIA
	LDA DDPIA+2
	ORA #$20
	CPY #$01
	BCC	*+4
	AND #$DF
	STA DDPIA+2
	RTS

; STEP TO TRACK 0

TZERO
	LDA MAXTRK	;max num tracks
	ADC #$08	;plus a few more
	STA TMP		;MAX times to step before abort
	LDA DDPIA+2
	AND #$FB	; 1111 1011
	BNE	TZERO3	; DIR=INWARDS

TZERO1
	DEC TMP
	BNE TZERO5
	SEC
	RTS
TZERO5	
	LDA	#2		; 0000 0010
	BIT	DDPIA	; TEST 'TRK0' PIN
	BNE	TZERO2	; AT TRACK 0
    LDA #0
    STA TRK
	STA TRKHI
	CLC
	RTS

TZERO2	
	LDA	DDPIA+2	; DIR=OUTWARDS (TO TRK0)
	ORA #$04
TZERO3	
	STA	DDPIA+2	; SET 'DIR' PIN
	JSR	DELAY2
	AND	#$F7	; 1111 0111
	STA	DDPIA+2	; SET 'STEP' PIN LOW
	JSR	DELAY2
	ORA	#8		; 0000 1000
	STA	DDPIA+2	; SET 'STEP' PIN HIGH
	LDX	#32 	; 40 MS
	JSR	DELAY
	BEQ	TZERO1	; ALWAYS

; STEP TO NEXT TRACK

TNEXT	
	LDA DDPIA+2
	AND	#$FB	; 1111 1011
				; DIR=INWARDS
	STA	DDPIA+2	; SET 'DIR' PIN
	JSR	DELAY2
	AND	#$F7	; 1111 0111
	STA	DDPIA+2	; SET 'STEP' LOW
	JSR	DELAY2
	ORA	#8		; 0000 1000
	STA	DDPIA+2	; SET 'STEP' HIGH
    INC TRK
	LDX	#32	    ; 40MS
	JMP	DELAY


; READ CURRENT TRACK TO BUFFER
; RETURNS Y=index of last byte read

RDTZERO
	JSR	INIT
	LDA DDPIA+2  ; PRESERVE DISK SEL BIT
	AND	#$7F	; 0111 1111
	STA	DDPIA+2	; LOAD DISK HEAD
	LDX	#$FF	; 320MS
	JSR	DELAY

	LDA #$80
	STA TMP
	LDX #$00

RDTRK1
	
	BIT	DDPIA	; WAIT for INDEX PULSE
	BPL	RDTRK8
	DEX
	BNE RDTRK1
	DEC TMP
	BNE RDTRK1
	SEC         ;error waiting for index
	RTS


RDTRK8
	LDX	ACIAVAL	; usually $58-DIVIDE BY 1, 8E1, RTS HIGH no IRQs
	LDA	#3
	LDY	#0		
RDTRK2			; now past index hole, reset acia, prepare to read data
	BIT	DDPIA	; WAIT END OF
	BPL	RDTRK2	; INDEX PULSE
	STA	DDACIA ; RESET ACIA
	STX	DDACIA ; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS
				 
		; READ UNTIL ALL PAGES DONE

RDTRK3
	BIT	DDPIA	; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL	RDTRK7
	LDA	DDACIA ;test byte ready?	
	LSR	A
	BCC	RDTRK3	  ;byte not ready?

RDTRK4
	LDA	DDACIA+1 ; READ BYTE
RDTRK9
	STA	(TDATA),Y
	INY
	BNE	RDTRK3
	INC	TDATA+1
	DEC	TRKCNT
	BNE	RDTRK3

RDTRK7
    JSR SETEOT
	LDA DDPIA+2
	ORA #$80	
	STA	DDPIA+2	; UNLOAD DISK HEAD
	CLC
	RTS

; READ TRACK 1-76 TO BUFFER
; RETURNS Y=index of last byte read

RDTRKU
	JSR	INIT
	LDA DDPIA+2  ; PRESERVE DISK SEL BIT
	AND	#$7F	; 0111 1111
	STA	DDPIA+2	; LOAD DISK HEAD
	LDX	#$FF	; 320MS
	JSR	DELAY

RDTRKU1	
	BIT	DDPIA	; WAIT START OF	INDEX PULSE
	BMI	RDTRKU1
	PHA
	PLA
	PHA 
	PLA
	PHA   ;delay
	PLA
	LDX	ACIAVAL	; usually $58-DIVIDE BY 1, 8E1, RTS HIGH no IRQs
	LDA	#3
	LDY	#0		
RDTRKU2			; now past index hole, reset acia, prepare to read data
	BIT	DDPIA	; WAIT END OF
	BPL	RDTRKU2	; INDEX PULSE
	
	STA	DDACIA ; RESET ACIA
	STX	DDACIA ; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS
				 
    LDX #$00
RDTRKUP
	; READ 1st 3 bytes as 8E1 OS65U uses $80, tkHi, tkLo
	BIT	DDPIA	; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL	RDTRKU5
	LDA	DDACIA ;test byte ready?	
	LSR	A
	BCC	RDTRKUP	  ;byte not ready?
	LDA	DDACIA+1 ; READ BYTE
	STA	(TDATA),Y
	CPX #$00
	BNE RDTYPE
	CMP #$43
	BNE *+3	  ;flag this is a 65D format 8E1 and continue reading there
	TAX
	CMP #$80
	BNE *+3
	TAX
RDTYPE
	INY
	CPY #$03
	BNE RDTRKUP
	CPX #$80
	BNE RDTRK3   ;didn't find OS65U start of track marker so continue in current format
	; once we've read 1st 3 bytes at 8E1, reset ACIA & set 8N1
	LDX #$07
RDELAY1
	DEX
	BNE RDELAY1	 ;delay same as 65u
	LDA #$03
	LDX #$54
	STA	DDACIA ; RESET ACIA
	STX	DDACIA ; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS

RDTRKU3
	BIT	DDPIA	; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL	RDTRKU5
	LDA	DDACIA ;test byte ready?	
	LSR	A
	BCC	RDTRKU3	  ;byte not ready?
	LDA	DDACIA+1 ; READ BYTE
	BMI RDTRKU3  ; skip byte(s) with hi bit set	(noise)
	BPL RDTRKU8

RDTRKU4			;finally read rest of track
	BIT	DDPIA	; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL	RDTRKU5
	LDA	DDACIA ;test byte ready?	
	LSR	A
	BCC	RDTRKU4	  ;byte not ready?
	LDA	DDACIA+1 ; READ BYTE
RDTRKU8
	STA	(TDATA),Y
	INY
	BNE	RDTRKU4
	INC	TDATA+1
	DEC	TRKCNT
	BNE	RDTRKU4

RDTRKU5			 ;now at end of track/index pulse start
	JSR SETEOT
	LDA DDPIA+2
	ORA #$80	
	STA	DDPIA+2	; UNLOAD DISK HEAD
	CLC
	RTS
	
SETEOT			;store count of last byte stored as last two bytes of sector data to be used when recreating disk image
	LDA MAXPAG  ;for $F page, we know offset if $EFE
	CLC
	ADC #>BUFFER
	STA PTRSTOR+1 ;calc page
	DEC PTRSTOR+1
	LDA #$FE
	STA PTRSTOR
	LDX #$00
	TYA
	STA (PTRSTOR,X)
	INC PTRSTOR
	LDA TDATA+1
	SEC
	SBC #>BUFFER
	STA (PTRSTOR,X)
	RTS
	

; DELAY 1.25MS PER LOOP @ 1MHZ
DELAY	
	LDY	#$F8
DELAY1	
	DEY
	BNE	DELAY1
	EOR	$FF,X
	DEX
	BNE	DELAY
	RTS

DELAY2	
	JSR	DELAY21
DELAY21	
	RTS

; Hard disk loader located at $FD00 for OSI 590/596 controller(?)
;
; Ports?/Mem at $EFFE, $EFFF
; Read  at $C280
; Write at $C200-$C207
; Read  at $e012
; ----------------------------------
; Disk Geometry	 74MB
; Cylinder (0 to 338)
; Track    (0 to 11)
; Sector   (0 to 4)
; sector is 3584 bytes long
;
; SA1004
; Track (0 to 255)
; Head (0 to 3)
;
;
; C200 = cylinder hi (bit 7) & track (bit0-6) 
; C201 = cylinder lo
; C202 = controller status
;               write                 read
;        bit 7  step dir
;        bit 6
;        bit 5 (head step 1->0)						
;        bit 4 1 = init? 			  1= (not ready?)
;        bit 3						  1= (not fail?)
;        bit 2                        1= (cmd ok?)
;      	 bit 1 disk sel               1= @Trk 0 (?)
;        bit 0 1=disk sel			  1= disk busy? ctrl OK when 1 ($x9 = OK)
; C202 = controller status / disk control load
;CD74/36 OSI 592 C202 READ DISK STATUS		   
;		 bit 7 (PW RST)						       
;		 bit 6 (PWR OK)     				       
;		 bit 5 MHD MulSel					     
;		 bit 4 MHD Cursense 				       
;		 bit 3 Ready        				       
;		 bit 2 Illegal Addr					      
;		 bit 1 Seek Late    				      
;      	 bit 0 Seek Complete				       
;
;        OSI 592 C202 WRITE /DISK CONTROL LOAD       
;        bit 7 m mode				  
;        bit 6 strobe late			  
;        bit 5 stobe early            
;        bit 4 restore                
;        bit 3 offset rev             
;        bit 2 offset fwd             
;        bit 1 drive sel hi           
;        bit 0 drive sel lo           
;
; -----------------------------------------------------------------------------------------------
;		OSI 596 
;        
;
; CD7/24 OSI 596 C202 READ DISK STATUS		 READ DISK STATUS     OSI 596 similar to 596
;		 bit 7 (wired 0) 					 0 = (always)		  After init reads $14
;		 bit 6 (wired 0)      				 0 = (always)		  
;		 bit 5 (wired 0) 					 0 = (always)		  
;		 bit 4 (wired 0)   				     0 = (always)         
;		 bit 3 Ready        				 1 = ready      	  
;		 bit 2 Write Fault					 1 = write fault	  
;		 bit 1 Track 0   				     1 = trk 0 sensed	   
;      	 bit 0 Seek Complete				 1 = seek complete     
;    before init reads $0F after reads $04
;
;        OSI 596 C202 WRITE DISK CONTRL LOAD       
;        bit 7 				  
;        bit 6 Step Dir		  
;        bit 5 Step	         
;        bit 4 Low I                 
;        bit 3 DS4  (device select)          
;        bit 2 DS3             
;        bit 1 DS2           
;        bit 0 DS1  


; C203 = start addr lo 
; C204 = start addr hi
; C205 = end addr lo
; C206 = end addr hi
;
; C207 = write                      read
;        bit 7 1=start operation  1=ctrlr busy
;        bit 6 1=write 0=read     0 
;        bit 5                    1
;        bit 4                    1
;        bit 3                    1
;        bit 2                    0
;        bit 1                    0
;        bit 0                    0
;
; C280 = access this to enable C20x register for ~10us
;
; $E010 - start of DMA memory
;
;  E000-E00E (zero out)
;  E00F = 1
; Sector offset table
; -------------------------------------
;          $C203   $C204  $C205  $C206
;       start addr lo,hi  end addr lo, hi
; sector 0 dv=$72 du=$00  dy=$62 dx=$08 
; sector 2 dy=$E1 du=$08  dy=$D1 dx=$10
;
; Boot track format
; code executes at $E018
; Sector is 3584/$E00 bytes long + overhead
; last two bytes contain checksum of track
; E013-E017 = block header

; E010 = $A1 (flag?)
; E011 = ($0)
; E012 = ($0) cyln hi?
; E013 = Cylinder
; E014 = Head
; E015 = Sector
; E016 = ?? flag ? $00 or $0F
; E017 = checksum of trk header (E010-E016)
; E018 - EE17 = sector data
; EE18, EE19 = checksum of sector data
;
; OS65U HD errors
;1 - Drive not ready
;2 - Seek timeout
;3 - Invalid unit number
;4 - Can't find cylinder 0
;5 - DMA failed to terminate
;6 - No data read
;7 - Sector unsafe
;8 - Header checksum
;9 - Cylinder mismatch
;10 - Track mismatch
;11 - Sector mismatch
;12 - Data field checksum
;13 - Status error
;14 - Unit out of service
;15 - Cylinder out of range

TDUMP
	LDA #$80
HDUMP=*+1
	BIT $01A9
	STA TMODE
	JSR ZHDRAM    ;is there RAM at $E000-EFFF?
	LDA $E000
	CMP #$E0
	BEQ BOOT3
	LDA $EF00
	CMP #$EF
	BNE BOOT1
BOOT3	
	JMP NOHD	;No RAM
BOOT1	
	JSR ANYKEY  ;press a key

BOOT2
	LDA #MAXPAGH  ;number of pages to read (0x10)
	STA MAXPAG
	LDA MAXBLK
	STA MAXTRK    ;number of tracks to read
	LDA MAXBLK+1
	STA MAXTRH
	LDA #$00
	STA TRK
	STA TRKHI
	STA CURSECT
	STA CURHEAD
	STA CURCYLN
	STA CURCYLH
	STA lastdata
	JSR INIT    ; INIT POINTERS, CLEAR BUFFER
	JSR INITHD
RHTOP
	JSR ZHDRAM	
	JSR RHARD
	JSR ERRHD
	JSR HDCOPY
	JSR	SAVMSG
	LDA #MAXPAGH   ;how many pages to read?  09/0F
	JSR	SAVBUF	
	BCS ROVER2
RHNXT	
	JSR NXTBLK
	BCC RHTOP
	;add code to reseek head
	
ROVER
	CMP #$00
	BEQ ROVER2
	
	JSR ERRHD   ;abort read, send 1 more block with error
	JSR HDCOPY
	LDA TRK
	LDX TRKHI
	CLC
	ADC #$01
	BCC *+3
	INX
	STX MAXTRH
	STA MAXTRK
	LDA #$01
	JSR	SAVBUF
	

ROVER2
	JSR INITHD ;zero head
	JSR DONEMSG
	JMP TOP
ZHDRAM         ;zero E000-EFFF
	LDA #$E0
	STA TDATA+1
	LDA #$00
	TAY
	STA TDATA
	LDX #$10
ZHDRA1
	STA (TDATA),Y
	INY
	BNE ZHDRA1
	INC TDATA+1
	DEX
	BNE ZHDRA1
	RTS

HDCOPY              ;copy HD memory to save buffer for send
	LDA	#<BUFFER
	STA	TDATA
	LDA	#>BUFFER
	STA	TDATA+1
	LDA #$00
	STA HDATA
	LDA #$E0
	STA HDATA+1
	LDY #$00
	LDX #$10  ;actually only $0F pages or less are filled ($E3E bytes max)
HDCPY1
	LDA (HDATA),Y
	STA (TDATA),Y
	INY
	BNE HDCPY1
	INC HDATA+1
	INC TDATA+1
	DEX
	BNE HDCPY1
	LDA	#>BUFFER
	STA	TDATA+1
	RTS


INITHD                                                                  
	LDA #$00      ;*init controller & track                                      
	LDX #$07                                                                     
	JSR CTRLOUT	   ;write $00 to C207 (DMA Reg Load)                                             
	STA HDTRK
REINIT                                                                    
	CLD                                                                          
	LDX #$07      ; Zero C207 down to C200
HINIT                                       
	LDA #$00	  ; C202 = 01                                                    
	CPX #$02                                                                     
	BNE NOSHFT                                                                    
	ROL A		   ;shift carry to bit 0
NOSHFT
	JSR CTRLOUT                                                                    
	DEX                                                                          
	BPL HINIT
	LDA $C202	   ;get status                                                    
	AND #$09       ;READY, SEEK DONE                                                              
	BNE REINIT	   ;ok? if not restart                                            
	LDA #$04	   ;* Wait for ctrl or HD? to get ready                           
	STA HCNT                                                                      
	LDY #$00
DCHECK1                                                                     
	LDA $C202	   ;get status                                                    
	AND #$02	   ;is bit 1 set?  (trk 0 sense?)                                 
	BEQ TRKOK	   ;if so continue                                                
	CLC            ;set up step to 0                                                              
	JSR STEPIT	   ;*write $21, $01 to C202 (step to 0)                                       
	LDX #$90       ;disk info says 1.5ms per step
DLOOP1                                                                     
	DEX                                                                          
	BNE DLOOP1	   ;delay some
DLOOP2                                                    
	DEX                                                                          
	BNE DLOOP2	   ;delay some more                                               
	DEY                                                                          
	BNE DCHECK1	   ;loop waiting for c202 bit 1                                   
	DEC HCNT                                                                      
	BPL DCHECK1	   ;try 4*256 times                                               
	BMI REINIT	   ;failed, jump back to reinit ctrlr                             
TRKOK
	TAY		       ;entry when ctrlr bit 1 = 1
TRKSTP1
	CPY HDTRK      ;contains target cyln                                                              
	BEQ TRKRDY	   ;carry set when register >= value                              
	JSR STEPIT	   ;write $61, $41 to $C202  (step up)                                     
	INY                                                                          
	BNE TRKSTP1
TRKRDY                                                                    
	LDA $C202	   ;wait for C202 bit 0 to clear                                          
	AND #$01                                                                     
	BNE TRKRDY 
    RTS
	
	
	
	
RHARD
	LDA #$04
	STA HCNT
RETRY
    JSR BLKREQ
    LDX #$07
	LDA #$80                                                                     
	JSR CTRLOUT	   ;write $80 to C207 (execute)
HLOOP1                                   
	LDA $C207	
	BPL HLOOP2
	CMP #$C2
	BNE HLOOP1	   ;wait until operation completes   disk ctrlr could disappear                             
	;disk ctrlr went bye-bye?
	LDA #$01
	JMP SECERR
HLOOP2	
    LDY #$06
	LDA #$00
    CLC
HDRCHK
	ADC $E010,Y
	DEY
	BPL HDRCHK
	CMP $E017
	BEQ HEADROK
	LDA #$08     ;8 - Header checksum
	BNE SECERR
	
HEADROK	
	LDA $E015
    CMP CURSECT
    BEQ SECTOK
    LDA #11     ;11 - Sector mismatch
	BNE SECERR

SECTOK
	LDA $E014
	CMP CURHEAD
	BEQ HEADOK
	LDA #10		;10 - Track mismatch
	BNE SECERR
	
HEADOK
	LDA $E013
	CMP CURCYLN
	BEQ CYLNOK
	LDA #9       ;9 - Cylinder mismatch
	BNE SECERR
	
CYLNOK	
	LDA #$18	   ;*chksum E018 thru EE17 compare to (EE18 lo, EE19 hi)          
	STA HDATA                                                                      
	LDA #$E0       ;addr hi                                                              
	STA HDATA+1                                                                      
	LDA #$0E       ;num pages                                                              
	STA HPAGE                                                                      
	LDA #$00                                                                     
	TAX                                                                          
	TAY                                                                          
	CLC
CHKSUM1                                                                          
	ADC (HDATA),Y                                                                  
	BCC HLOOP6                                                                    
	INX                                                                          
	BEQ HLOOP6                                                                    
	CLC 
HLOOP6                                                                         
	INY                                                                          
	BNE CHKSUM1                                                                   
	INC HDATA+1                                                                      
	DEC HPAGE                                                                      
	BNE CHKSUM1                                                                    
	STA HDATA                                                                      
	STX HDATA+1                                                                      
	CMP $EE18                                                                    
	BNE CHKERR                                                                    
	CPX $EE19                                                                    
	BNE CHKERR 
    CLC
    LDA #$00	
	RTS
CHKERR
	LDA #$0C  ;Checksum err
SECERR
	INC $EFFF
	DEC HCNT
	BMI *+5
	JMP RETRY
	SEC
	RTS
	
STEPIT	         ;STEP HEAD  carry set=step out                                                                  
	PHP		     ;*write to $C202 $21 then $01  to step in (lower)                                 
	LDA #$00	 ;if carry set write $61 then $41 to step out (higher)                                  
	ROR A		                                     
	ROR A                                                                        
	PLP                                                                          
	ORA #$21                                                                     
	JSR CTRLOUT2                                                                    
	EOR #$20
CTRLOUT2                                                                     
	LDX #$02	   ;*Write byte in A to $C202 
CTRLOUT                                    
	PHP           ;*Write byte in A to controller at X offset                    
	SEI                                                                          
	BIT $C280     ;make controller visible                                                            
	STA $C200,X
	PLP                                                                          
	RTS
; these values are loaded into the start and end 20 bit counters on the controller board (addr bits 16-19 = 0)
SECS = * ;sector start offsets lo,hi
	.BYTE $72, $00  ;sector1
	.BYTE $E1, $08  ;sector 2 ($7f from end sector value???)
	
SECE =*	 ;sector end offsets lo,hi  okay I don't know why these offset only equal $7F0 difference but thats what OSI did
    .BYTE $62, $08  ;end sector1
	.BYTE $D1, $10  ;end sector2

MAXBLK=*  ; maximum block to transfer
	;.BYTE $F8, $07  ;max blk for CD-7 is 2sect*4head*256cyln (*3584 bytes = 7340032)+overhead  (7314943)
    .BYTE $00, $08 
	
BLKREQ    ;select 1st or 2nd sector to read based on block #
	LDA TRK
	AND #$01
	STA CURSECT
	ASL A 
	TAY 
	LDA SECS,Y
	LDX SECS+1,Y
BLKSEL=*
	PHP 
	SEI 
	BIT $C280
	STA $C203  ;load starting addr
	STX $C204
	LDA SECE,Y
	LDX SECE+1,Y
	BIT $C280
	STA $C205  ;load ending addr
	STX $C206
	
	LDA TRK
	AND #$07
	LSR A
	STA CURHEAD
	BIT $C280 ; set head sel
	STA $C200
HEDSEL
	JSR DELAY
	PLP 
	RTS
	
NXTBLK        ;go to next block on hard drive
	LDA TRK
	LDX TRKHI
	CLC
	ADC #$01
	BCC *+3
	INX
	STX TRKHI
	STA TRK
	CMP MAXBLK
	BNE TSTEP
	CPX MAXBLK+1
	BNE TSTEP
	LDA #$00 ;done - last block, no err
	SEC
	RTS
TSTEP           ;step to next cylinder every 8 blocks
    LDY #$00
    AND #$07     ;every 8 block, step to next cylinder
	CMP #$01
	BCS TDONE+2  ;no step needed
	SEC
	JSR STEPIT	   ;step higher
	INC CURCYLN
	BNE TSTEP4
	INC CURCYLH
TSTEP4
	LDA #$04	   ;* Wait for ctrl or HD? to get ready                           
	STA HCNT 
TCHECK	
    LDA $C202
    AND #$01
    BEQ TDONE	
TSTEP0
	LDY #$00	
	LDX #$90       ;disk info says 1.5ms per step
TSTEP1                                                                     
	DEX                                                                          
	BNE TSTEP1	   ;delay some
TSTEP2                                                    
	DEX                                                                          
	BNE TSTEP2	   ;delay some more                                               
	DEY                                                                          
	BNE TCHECK	   ;loop waiting for c202 bit 1                                   
	DEC HCNT                                                                      
	BPL TCHECK	   ;try 4*256 times
	LDA #$02       ;error stepping
	SEC
	RTS

TDONE
	LDA #$00
	CLC
	RTS

HMSG1=*
    .BYTE 'ERR',0
HDMESG  ;store error in HD memory for transfer
	STA $E004
	CMP #$00
	BEQ HDMESG2
    LDX #$02
HDMESG1
	LDA HMSG1,X
	STA $E000,X
	DEX
	BPL HDMESG1
HDMESG2
	RTS

 	
; PRINT IN-LINE STRING

PRINT	
	PLA
	STA	PTRSTOR
	PLA
	STA	PTRSTOR+1
	LDY	#1
PRINT1	
	LDA	(PTRSTOR),Y
	BEQ	PRINT2
	JSR	OUTPUT
	INY
	BNE	PRINT1
	INC PTRSTOR+1
	BNE PRINT1
PRINT2	
	TYA
	SEC
	ADC	PTRSTOR
	STA	PTRSTOR
	BCC	PRINT3
	INC	PTRSTOR+1
PRINT3
    CLC	
	JMP	(PTRSTOR)

; PRINT HEX WORD (A,X)

PHEX	
	JSR	PHEXA
	TXA

; PRINT HEX BYTE (A)
PHEXA	
	PHA
	LSR	A
	LSR	A
	LSR	A
	LSR	A
	JSR	PHEXA1
	PLA
PHEXA1	
	AND	#$F
	ORA	#'0'
	CMP	#'9'+1
	BMI	PHEXA2
	CLC
	ADC	#7
PHEXA2	
	JMP	OUTPUT

; PRINT DECIMAL (A)
PDECA	
	TAX
PDECX
	LDA	#0

; PRINT DECIMAL (A,X) (hi, lo)  uses A,X,Y

PDEC	
	STX	PDEC8      ;binary lo
	STA	PDEC8+1    ;binary hi
	LDA	#0
	LDX	#2
PDEC1	
	STA	PDEC10,X   ;zero result
	DEX
	BPL	PDEC1
	LDY	#16        ;number of source bits
	PHP            ;save cpu flags
	SEI            
	SED
PDEC2	
	ASL	PDEC8      ;shift out 1 bit
	ROL	PDEC8+1
	LDX	#2
PDEC3	
	LDA	PDEC10,X   ;add into result, propagate carry
	ADC	PDEC10,X
	STA	PDEC10,X
	DEX
	BPL	PDEC3
	DEY
	BNE	PDEC2      ;loop till done
	PLP
	LDX	#6         ;print result
	LDA	#0
	STA	PDEC9
PDEC4	
	LDY	#3
	LDA	#0
PDEC5	
	ASL	PDEC10+2  ;shift 4 bits of result out to A
	ROL	PDEC10+1
	ROL	PDEC10
	ROL	A
	DEY
	BPL	PDEC5
	TAY          ;lead zero blanking
	BNE	PDEC6
	CPX	#1       ;unless last digit
	BEQ	PDEC6
	LDY	PDEC9
	BEQ	PDEC7
PDEC6	
    INC PDEC9
	ORA	#$30
	JSR	OUTPUT
PDEC7	
    DEX	
	BNE	PDEC4
	RTS

CRLF	
	LDA	#CR
	JSR	OUTPUT
	LDA	#LF
	JMP	OUTPUT

ANYKEY	
	JSR	PRINT
	.BYTE	CR,LF
	.BYTE	'PRESS ANY KEY WHEN READY',CR,LF,0
	JMP	INKEY

READMSG
	LDA #$0D
	BIT TMODE
	BMI READMS1
	JSR OUTPUT
	LDA #$0A
READMS1	
    JSR OUTPUT
	JSR	PRINT
	.BYTE	';READING BLOCK ',0
    RTS 
DONEMSG	
	JSR	PRINT
	.BYTE	CR,LF,CR,LF
	.BYTE	';DONE - PRESS ANY KEY',0
	JMP	INKEY

SAVMSG
	BIT TMODE
	BMI SAVMS0
	LDA USEXMOD
	BNE SAVMEND   ;no msg with xmodem
SAVMS0
	JSR READMSG
	LDX TRK
	LDA TRKHI
	BEQ SAVMS1
	JMP	PDEC
SAVMS1
	JSR PDECX
SAVMEND
	RTS	

NOHD
	JSR PRINT
	.BYTE CR,LF
	.BYTE 'NO HARD DISK CONTROLLER DETECTED',CR,LF,0
	JSR ANYKEY
	JMP TOP
ERRHD
	CMP #$00
	BEQ ERRHD2
	PHA
    LDA USEXMOD
	BNE ERRHD1
	JSR PRINT
	.BYTE CR,LF
	.BYTE ';HD Error #',0
	PLA
	PHA
	JSR PDECA
	JSR CRLF
ERRHD1
	PLA
	JSR HDMESG
	
ERRHD2
	RTS

;Track 0 format
;10 millisecond delay after index hole
;0,1 load address of the track in hi,lo form
;2, page count of how much data is written on track 0, usually 8
;3+ data

;Track 1-n Format
;0,1 -a 2 byte start code $43, $57
;2,  -BCD track number
;3,  -track type code (always $58)

; sector data
;1,  -sector start code $76
;1,  -sector number in binary
;1   -sector length (#pages) in binary
;x   -sector data (1-8/12 pages)
;2   -end of sector mark $47, $53
; next sector

;track 1-76 format
; 8E1 
; 0x80, tklo, tkhi, <reset ACIA to 8N1> 0x1F ... data ...


DRWMENU
	 JSR Flushm
	 LDX #$16		;is the disk controller present?
DRWCHK
	 LDA DDPIA,X
	 CMP #>DDPIA
     BNE DRWMEN0
	 DEX
	 BPL DRWCHK
	 JSR PRINT
	 .BYTE $D,$A,' *NO DISK CTRLR!',0
	 
DRWMEN0
	 JSR PRINT
	.BYTE $D,$A,$D,$A
	.BYTE 'OSI C3 Serial Disk Dump ',$D,$A          
	.BYTE '----------------------- ',$D,$A          
	.BYTE '1. Dump Floppy',$D,$A
	.BYTE '2. Dump CD-7 Hard Drive',$D,$A
	.BYTE '3. Select Floppy Drive',$D,$A 
    .BYTE '4. Scan HD for errors',$D,$A	
	.BYTE '5. Set Serial Divisor',$D,$A
	.BYTE '6. Set Xmodem/Hex xfer',$D,$A 
	.BYTE '7. Exit',$D,$A,$D,$A
	.BYTE 'Drv=',$0
	LDA DRVNUM
	CLC
	ADC #$41
	JSR	OUTPUT
	LDA	MAXPAG
	CMP #$0A
	BCS DRWMN1
	JSR PRINT
	.BYTE '/5.25',0
	BCC DRWMN2
DRWMN1
	JSR PRINT
	.BYTE '/8 ',0
DRWMN2
	JSR PRINT
	.BYTE ' Baud@1',0
   	LDA ACIACTL
	LSR A
	BCS DRWMN3
	LDA #'6
	JSR OUTPUT
DRWMN3
	JSR PRINT
	.BYTE 'x ',0
DRWMN4
	BIT USEXMOD
	BPL DRWMN5
	BVC DRWMN8
	JSR PRINT
	.BYTE 'Xmdm ',0
	BCC DRWMN6
DRWMN8
	JSR PRINT
	.BYTE 'Xmdm/crc ',0
	BCC DRWMN6
DRWMN5	
	JSR PRINT
	.BYTE 'Hex ',0
DRWMN6
	JSR PRINT
	 .BYTE $D,$A,'--->',0            
	RTS


INZACIA
    LDA #$03	;reset ACIA
	STA $FC00
	LDA ACIACTL	;
	STA $FC00
	RTS

Get_chr
FRACIANW  ; read from ACIA no wait	carry clear when no data
	LDA $FC00
	LSR A
	BCC FRAC1NW
	LDA $FC01
FRAC1NW
	RTS


; get char from ACIA waiting
FRACIA   	 ;read from ACIA carry set on abort return value in A
	LDA $FC00
	LSR A    
	BCC FRACIA 
	LDA $FC01
	CLC      
	RTS


OUTPUT=*		; output to ACIA only on C3
Put_Chr
TOACIA    
	PHA
TOACIA1     
	LDA $FC00   ;wait for TxEmpty
	LSR A     
	LSR A     
	BCC TOACIA1   
	PLA       
	STA $FC01 
	RTS       

INKEY
	JMP FRACIA  ;Serial system


;----------------------------------------------------------------------------
;
; packet is 128 bytes, last packet is zero padded 
; packet consists of <SOH=$01><blk# (starting at 1)><255-block#>< 128 data bytes><crclo><crc hi> for 133 bytes
; "C" is used to acknowledge & start transfer

; XMODEM/CRC Sender for the 65C02
;
; By Daryl Rictor Aug 2002
;
; A simple file transfer program to allow upload from the SBC to a 
; console device utilizing the x-modem/CRC transfer protocol.  Requires just
; under 1k of either RAM or ROM, 132 bytes of RAM for the receive buffer,
; and 8 bytes of zero page RAM for variable storage.
;
;**************************************************************************
; This implementation of XMODEM/CRC does NOT conform strictly to the 
; XMODEM protocol standard in that it (1) does not accurately time character
; reception or (2) fall back to the Checksum mode.

; (1) For timing, it uses a crude timing loop to provide approximate
; delays.  These have been calibrated against a 1MHz CPU clock.  I have
; found that CPU clock speed of up to 5MHz also work but may not in
; every case.  Windows HyperTerminal worked quite well at both speeds!
;
; (2) Most modern terminal programs support XMODEM/CRC which can detect a
; wider range of transmission errors so the fallback to the simple checksum
; calculation was not implemented to save space.
;**************************************************************************
;
; Files uploaded via XMODEM-CRC will have the load address contained in
; the first two bytes in little-endian format:  
;  FIRST BLOCK
;     offset(0) = lo(load start address),
;     offset(1) = hi(load start address)
;     offset(2) = data byte (0)
;     offset(n) = data byte (n-2)
;
; Subsequent blocks
;     offset(n) = data byte (n)
;
; One note, XMODEM send 128 byte blocks.  If the block of memory that
; you wish to save is smaller than the 128 byte block boundary, then
; the last block will be padded with zeros.  Upon reloading, the
; data will be written back to the original location.  In addition, the
; padded zeros WILL also be written into RAM, which could overwrite other
; data.   
;
;-------------------------- The Code ----------------------------
;
; zero page variables (adjust these to suit your needs)
;
;
;xmdata      =   $35
;lastblk		=	xmdata			; flag for last block
;blkno		=	xmdata+1		; block number 
;Xerrcnt		=	xmdata+2		; error counter 10 is the limit

;crc			=	xmdata+3		; CRC lo byte  (two byte variable)
;crch		=	xmdata+4		; CRC hi byte  

;ptr			=	xmdata+5		; data pointer (two byte variable)
;ptrh		=	xmdata+6		;   "    "

;eofp		=	xmdata+7		; end of file address pointer (2 bytes)
;eofph		=	xmdata+8		;  "	"	"	"

;Xretry		=	xmdata+9		; retry counter 
;Xretry2		=	xmdata+10		; 2nd counter
;lastdata     =   xmdata+11       ;flag buffer of file

;
;
; non-zero page variables and buffers
;
;
;Rbuff		=	$4000      	; temp 133 byte receive buffer 
;					;(place anywhere, page aligned)
;
;
;  tables and constants
;
;
; The crclo & crchi labels are used to point to a lookup table to calculate
; the CRC for the 128 byte data blocks.  There are two implementations of these
; tables.  One is to use the tables included (defined towards the end of this
; file) and the other is to build them at run-time.  If building at run-time,
; then these two labels will need to be un-commented and declared in RAM.
;
;crclo		=	$7D00      	; Two 256-byte tables for quick lookup
;crchi		= 	$7E00      	; (should be page-aligned for speed)
;
;
;


;
;^^^^^^^^^^^^^^^^^^^^^^ Start of Program ^^^^^^^^^^^^^^^^^^^^^^
;
; Xmodem/CRC download routine
; By Daryl Rictor, August 8, 2002
;
; v0.1  code creation
;
;
;		*= 	$7800		; Start of program (adjust to your needs)
;
; Enter this routine with the beginning address stored in the zero page address
; pointed to by ptr & ptrh and the ending address stored in the zero page address
; pointed to by eofp & eofph.
;
; 1K option: 1K packets allowed A STX (02) replaces SOK (01) at the beginning of a block
; sender can change packet lengths in stream as long as previous packet has been ack'd
; 
;
XResume
		bit	lastdata
		bvc	XModemStart	 ;bit 6 = 0, means its 1st block
		ldx	Xsavem
		ldy	#$00
		STY Xerrcnt    
		;JSR GetByte
		JSR FlushQ		;MAC/Linux lrzsz sends us NAKs when we we're busy
		CMP #CAN
		BEQ XAbort
		CMP #ESC
		BEQ XAbort
		
		BNE	LdBuff1
XAbort
		Jmp GotAbort

;XMODEM Send 		
XModemStart
		lda	#$40		;bit tst V = 1, means not 1st block next time
		sta	lastdata
		jsr	PrintMsg	; send prompt and info
		lda	#$00		;
		sta	Xerrcnt		; error counter set to 0
		sta	lastblk		; set flag to false
		lda	#$01		;
		sta	blkno		; set block # to 1
Wait4CRC
		lda	#$ff		; 3 seconds
		sta	Xretry2		;
		jsr	GetByte		;
		bcc	Wait4CRC	; wait for something to come in...
		cmp	#'C			; is it the "C" to start a CRC xfer?
		beq	SetCRC		; yes
		cmp #NAK
		BNE Wait1
		LDA #$C0
		.BYTE $2C
SetCRC
		LDA #$80
		STA USEXMOD		;receiver wants std xmodem
		JMP SetstAddr
Wait1
		cmp #CAN
		beq *+6
		cmp	#ESC		; is it a cancel? <Esc> Key
		bne	Wait4CRC	; No, wait for another character
		jmp	PrtAbort	; Print abort msg and exit
SetstAddr
		ldy	#$00		; init data block offset to 0
		
		lda	#$01		; manually load Blk #1	
		sta	Rbuff		; into 1st byte
		lda	#$FE		; load 1's comp of block #	
		sta	Rbuff+1		; into 2nd byte
	;when transferring disk images, we don't send load address
		;ldx	#$04		; preload X to Receive buffer
		;lda	ptr			; load low byte of start address		
		;sta	Rbuff+2		; into 3rd byte	
		;lda	ptr+1		; load hi byte of start address		
		;sta	Rbuff+3		; into 4th byte
		ldx	#$02        ;our buffer only has 2 byte preloaded now
		jmp	Ldbuff1		; jump into buffer load routine

LdBuffer
		lda	lastblk		; Was the last block sent?
		beq	LdBuff0		; no, send the next one	
		jmp Done		; yes, we're done
LdBuff0
		ldx	#$02		; init pointers
		ldy	#$00		;
		inc	Blkno		; inc block counter
		lda	Blkno		; 
		sta	Rbuff		; save in 1st byte of buffer
		eor	#$FF		; 
		sta	Rbuff+1		; save 1's comp of blkno next

LdBuff1
		lda	(ptr),y		; save 128 bytes of data
		sta	Rbuff,x		;
LdBuff2
		lda	ptr+1 	;
		cmp	EOFPH		;
		bcc	LdBuff4		;
		lda	ptr		;
		cmp	EOFP		; Are we at the last address?
		bcc	LdBuff4		; no, inc pointer and continue

		bit	lastdata	; set when last transfer block
		bmi	LdBuf2b
		stx	Xsavem      ;save output buffer pos for next time we are called
		clc
		rts
LdBuf2b                 ;no more blocks to send ; if we're at the start of a block finish
		CPX #$02        ;otherwise continue to pad to end
		BNE LdBuf2a
LdBuf2c
		LDA #EOT
		jsr	Put_Chr
		jsr	GetByte		
		CMP #ACK
		BEQ LdBuf2d
		CMP #NAK
		BEQ LdBuf2c
		CMP #CAN
		BEQ LdBuf2d
		CMP #ESC
		BNE LdBuf2c
LdBuf2d
		JMP Done
LdBuf2a
		inc	lastblk		; Yes, Set last byte flag
LdBuff3
		inx			;
		cpx	#$82		; Are we at the end of the 128 byte block?
		beq	DoCRC		; Yes, calc CRC
		lda	#$00		; Fill rest of 128 bytes with $00
		sta	Rbuff,x		;
		beq	LdBuff3		; Branch always

LdBuff4
		inc	ptr		; Inc address pointer
		bne	LdBuff5		;
		inc	ptr+1		;
LdBuff5
		inx			;
		cpx	#$82		; last byte in block?
		bne	LdBuff1		; no, get the next

DoCRC
		LDY #$82
		JSR CalcCRC
		lda	crch		; save Hi byte of CRC to buffer
		sta	Rbuff,y		;
		iny			;
		lda	crc			; save lo byte of CRC to buffer
		sta	Rbuff,y		;
Resend
		ldx	#$00		;
		lda	#SOH		; Send start block command
		jsr	Put_Chr		;
SendBlk
		lda	Rbuff,x		; Send 133 bytes in buffer to the console
		jsr	Put_Chr	;

		inx			;
		CPX #$84
		BCC SendBlk		
		BIT USEXMOD		;using CRC mode?
		BVS sendbl2		;no checksum, so branch
sendbl1
		cpx	#$84		; last byte?
		bne	SendBlk		; no, get next
sendbl2
		lda	#$FF		; yes, set 3 second delay 
		sta	Xretry2		; and
		jsr	GetByte		; Wait for Ack/Nack
		bcc	Seterror	; No chr received after 3 seconds, resend
		cmp	#NAK		; 
		beq	Seterror	; NAK, inc errors and resend
		cmp	#ESC		;
		beq	PrtAbort	; Esc pressed to abort
		cmp	#ACK		; Chr received... is it:
		bne SetError
		LDA #$00		;reset error count
		STA Xerrcnt
		JMP LdBuffer	; ACK, send next block				
Seterror
		inc	Xerrcnt		; Inc error counter
		lda	Xerrcnt		; 
		cmp	#$0A		; are there 10 errors in a row? (Xmodem spec for failure)
		bcc	Resend		; no, resend block
		JSR PRINT
		.byte 	CAN,CAN,CAN,'Errors Exceeded!',CR,LF,0
		sec
		rts		
PrtAbort
		jsr	Flush		; yes, too many errors, flush buffer,
		JSR PRINT
		.byte 	CAN,CAN,CAN,'Abort Received!',CR,LF,0
		sec
		rts		
Done
		jmp	Print_Good	; All Done..Print msg and exit

GotAbort
		JSR PrtAbort
		lda	#ABORTRECV		; Error code in "A" of desired
		sec
		rts		
StartBlk
		lda	#$FF		; 
		sta	Xretry2		; set loop counter for ~3 sec delay
		jsr	GetByte		; get first byte of block
		bcc	StartBlk	; timed out, keep waiting...
;
;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;
; subroutines
;
;					;
GetByte	
        lda #$00		; wait for chr input and cycle timing loop
		sta	Xretry		; set low value of timing loop
StartCrcLp	
		jsr	Get_chr 	; get chr from serial port, don't wait 
		bcs	GetByte1	; got one, so exit
		EOR	$FF,X       ; waste some time
		EOR	$FF,X	
		dec	Xretry		; no character received, so dec counter
		bne	StartCrcLp	;
		dec	Xretry2		; dec hi byte of counter
		bne	StartCrcLp	; look for character again
		clc			    ; if loop times out, CLC, else SEC and return
GetByte1
		rts			    ; with character in "A"
;
Flushm    ;flush lite
	    lda #$20
		.BYTE $2c   ;bit
Flush		
		lda	#$70		; flush receive buffer
		sta	Xretry2		; flush until empty for ~1 sec.
Flush1		
		jsr	GetByte		; read the port
		bcs	Flush		; if chr recvd, wait for another
		rts			; else done
;
FlushQ
		jsr Get_chr
		bcc FlushQ1
		CMP #NAK
		BEQ FlushQ
		RTS
FlushQ1
		LDA #$00
		rts

PrintMsg
		JSR PRINT
		.BYTE	'Begin XMODEM/',0
		BIT USEXMOD
		BVS Printm1
		JSR PRINT
		.BYTE 'CRC',0
		BCC Printm2
Printm1
		JSR PRINT
		.BYTE 'Checksum',0
Printm2
		JSR PRINT
		.BYTE   ' transfer.  Press <ESC> to abort...'
		.BYTE  	CR, LF, 0		
		RTS

Print_Good
		JSR PRINT
		.BYTE	EOT,EOT,EOT,CR,LF,CR,LF   ; send end of file three times
		.BYTE 	'Transfer Complete!'
		.BYTE  	CR, LF, 0
		CLC	
		RTS

CalcChk
	LDY #$02
	LDA #$00
CalcCh1
    CLC
	ADC	Rbuff,y		;
	iny			;
	cpy	#$82		; done yet?
	bne	CalcCh1
	STA crch		;value of checksum byte use when creating send block
	RTS		
;=========================================================================
;
;
;  CRC subroutines 
;
;
CalcCRC
		BIT USEXMOD
		BVS CalcChk
		lda	#$00		; yes, calculate the CRC for the 128 bytes
		sta	crc		;
		sta	crch		;
		ldy	#$02		;
CalcCRC1
		lda	Rbuff,y		;
		jsr	UpdCRC		;		
		iny			;
		cpy	#$82		; done yet?
		bne	CalcCRC1	; no, get next
		RTS

UpdCrc		
		eor 	crc+1 		; Quick CRC computation with lookup tables
		tax		 	; updates the two bytes at crc & crc+1
		lda 	crc		; with the byte send in the "A" register
		eor 	CRCHI,X
		sta 	crc+1
		lda 	CRCLO,X
		sta 	crc
		rts
;
;
;======================================================================
;  I/O Device Specific Routines
;
;  Two routines are used to communicate with the I/O device.
;
; "Get_Chr" routine will scan the input port for a character.  It will
; return without waiting with the Carry flag CLEAR if no character is
; present or return with the Carry flag SET and the character in the "A"
; register if one was present.
;
; "Put_Chr" routine will write one byte to the output port.  Its alright
; if this routine waits for the port to be ready.  its assumed that the 
; character was send upon return from this routine.
;
; Here is an example of the routines used for a standard 6551 ACIA.
; You would call the ACIA_Init prior to running the xmodem transfer
; routine.
;
;ACIA_Data	=	$7F70		; Adjust these addresses to point 
;ACIA_Status	=	$7F71		; to YOUR 6551!
;ACIA_Command	=	$7F72		;
;ACIA_Control	=	$7F73		;
;
;ACIA_Init      	lda	#$1F           	; 19.2K/8/1
;               	sta	ACIA_Control   	; control reg 
;               	lda	#$0B           	; N parity/echo off/rx int off/ dtr active low
;               	sta	ACIA_Command   	; command reg 
;               	rts                  	; done
;
; input chr from ACIA (no waiting)
;
;Get_Chr		clc			; no chr present
;               	lda	ACIA_Status     ; get Serial port status
;               	and	#$08            ; mask rcvr full bit
;              	beq	Get_Chr2	; if not chr, done
;               	Lda	ACIA_Data       ; else get chr
;	       	sec			; and set the Carry Flag
;Get_Chr2    	rts			; done
;
; output to OutPut Port
;
;Put_Chr	   	PHA                     ; save registers
;Put_Chr1     	lda	ACIA_Status     ; serial port status
;              	and	#$10            ; is tx buffer empty
;               	beq	Put_Chr1        ; no, go back and test it again
;               	PLA                     ; yes, get chr to send
;               	sta	ACIA_Data       ; put character to Port
;               	RTS                     ; done
;=========================================================================

;
; Alternate solution is to build the two lookup tables at run-time.  This might
; be desirable if the program is running from ram to reduce binary upload time.
; The following code generates the data for the lookup tables.  You would need to
; un-comment the variable declarations for crclo & crchi in the Tables and Constants
; section above and call this routine to build the tables before calling the
; "xmodem" routine.
;
;MAKECRCTABLE
;		ldx 	#$00
;		LDA	#$00
;zeroloop	sta 	crclo,x
;		sta 	crchi,x
;		inx
;		bne	zeroloop
;		ldx	#$00
;fetch		txa
;		eor	crchi,x
;		sta	crchi,x
;		ldy	#$08
;fetch1		asl	crclo,x
;		rol	crchi,x
;		bcc	fetch2
;		lda	crchi,x
;		eor	#$10
;		sta	crchi,x
;		lda	crclo,x
;		eor	#$21
;		sta	crclo,x
;fetch2		dey
;		bne	fetch1
;		inx
;		bne	fetch
;		rts
;
; The following tables are used to calculate the CRC for the 128 bytes
; in the xmodem data blocks.  You can use these tables if you plan to 
; store this program in ROM.  If you choose to build them at run-time, 
; then just delete them and define the two labels: crclo & crchi.
;
; low byte CRC lookup table (should be page aligned)

*= *+255/256*256 ;page align the following tables
crclo
 .byte $00,$21,$42,$63,$84,$A5,$C6,$E7,$08,$29,$4A,$6B,$8C,$AD,$CE,$EF
 .byte $31,$10,$73,$52,$B5,$94,$F7,$D6,$39,$18,$7B,$5A,$BD,$9C,$FF,$DE
 .byte $62,$43,$20,$01,$E6,$C7,$A4,$85,$6A,$4B,$28,$09,$EE,$CF,$AC,$8D
 .byte $53,$72,$11,$30,$D7,$F6,$95,$B4,$5B,$7A,$19,$38,$DF,$FE,$9D,$BC
 .byte $C4,$E5,$86,$A7,$40,$61,$02,$23,$CC,$ED,$8E,$AF,$48,$69,$0A,$2B
 .byte $F5,$D4,$B7,$96,$71,$50,$33,$12,$FD,$DC,$BF,$9E,$79,$58,$3B,$1A
 .byte $A6,$87,$E4,$C5,$22,$03,$60,$41,$AE,$8F,$EC,$CD,$2A,$0B,$68,$49
 .byte $97,$B6,$D5,$F4,$13,$32,$51,$70,$9F,$BE,$DD,$FC,$1B,$3A,$59,$78
 .byte $88,$A9,$CA,$EB,$0C,$2D,$4E,$6F,$80,$A1,$C2,$E3,$04,$25,$46,$67
 .byte $B9,$98,$FB,$DA,$3D,$1C,$7F,$5E,$B1,$90,$F3,$D2,$35,$14,$77,$56
 .byte $EA,$CB,$A8,$89,$6E,$4F,$2C,$0D,$E2,$C3,$A0,$81,$66,$47,$24,$05
 .byte $DB,$FA,$99,$B8,$5F,$7E,$1D,$3C,$D3,$F2,$91,$B0,$57,$76,$15,$34
 .byte $4C,$6D,$0E,$2F,$C8,$E9,$8A,$AB,$44,$65,$06,$27,$C0,$E1,$82,$A3
 .byte $7D,$5C,$3F,$1E,$F9,$D8,$BB,$9A,$75,$54,$37,$16,$F1,$D0,$B3,$92
 .byte $2E,$0F,$6C,$4D,$AA,$8B,$E8,$C9,$26,$07,$64,$45,$A2,$83,$E0,$C1
 .byte $1F,$3E,$5D,$7C,$9B,$BA,$D9,$F8,$17,$36,$55,$74,$93,$B2,$D1,$F0 

; hi byte CRC lookup table (should be page aligned)
;		*= $7d00
crchi
 .byte $00,$10,$20,$30,$40,$50,$60,$70,$81,$91,$A1,$B1,$C1,$D1,$E1,$F1
 .byte $12,$02,$32,$22,$52,$42,$72,$62,$93,$83,$B3,$A3,$D3,$C3,$F3,$E3
 .byte $24,$34,$04,$14,$64,$74,$44,$54,$A5,$B5,$85,$95,$E5,$F5,$C5,$D5
 .byte $36,$26,$16,$06,$76,$66,$56,$46,$B7,$A7,$97,$87,$F7,$E7,$D7,$C7
 .byte $48,$58,$68,$78,$08,$18,$28,$38,$C9,$D9,$E9,$F9,$89,$99,$A9,$B9
 .byte $5A,$4A,$7A,$6A,$1A,$0A,$3A,$2A,$DB,$CB,$FB,$EB,$9B,$8B,$BB,$AB
 .byte $6C,$7C,$4C,$5C,$2C,$3C,$0C,$1C,$ED,$FD,$CD,$DD,$AD,$BD,$8D,$9D
 .byte $7E,$6E,$5E,$4E,$3E,$2E,$1E,$0E,$FF,$EF,$DF,$CF,$BF,$AF,$9F,$8F
 .byte $91,$81,$B1,$A1,$D1,$C1,$F1,$E1,$10,$00,$30,$20,$50,$40,$70,$60
 .byte $83,$93,$A3,$B3,$C3,$D3,$E3,$F3,$02,$12,$22,$32,$42,$52,$62,$72
 .byte $B5,$A5,$95,$85,$F5,$E5,$D5,$C5,$34,$24,$14,$04,$74,$64,$54,$44
 .byte $A7,$B7,$87,$97,$E7,$F7,$C7,$D7,$26,$36,$06,$16,$66,$76,$46,$56
 .byte $D9,$C9,$F9,$E9,$99,$89,$B9,$A9,$58,$48,$78,$68,$18,$08,$38,$28
 .byte $CB,$DB,$EB,$FB,$8B,$9B,$AB,$BB,$4A,$5A,$6A,$7A,$0A,$1A,$2A,$3A
 .byte $FD,$ED,$DD,$CD,$BD,$AD,$9D,$8D,$7C,$6C,$5C,$4C,$3C,$2C,$1C,$0C
 .byte $EF,$FF,$CF,$DF,$AF,$BF,$8F,$9F,$6E,$7E,$4E,$5E,$2E,$3E,$0E,$1E
;
;
; End of Xmodem
;

BUFFER	=	*
*=*+$1000 ;space for track buffer


.END

